//INPUT:
//xquery.in -- contains #rows,#cols,initial position, and map

//OUTPUT:
//query.log -- log of calls to this library

unit query;
interface
    procedure Right;
    procedure Left;
    function Forward:integer;
implementation
var
 grid: array[0..60,0..60] of char;
 curRow,curCol,initialRow,initialCol,rows,cols,headingr,headingc : longint;
 initialized : longint ;
 debugf : text;

 (*
//expecting input of the form
//5 6 2 3 S    //rows cols curRow curCol orientation    // 1 <= curRow <= rows
//X.X...                                            // 1 <= curCol <= cols
//X....X
//X....X
//X....X
//......

// direction  (headingr, headingc)
// -------------------------------
// N (-1,0)
// E (0,1)
// S (1,0)
// W (0,-1)
*)

procedure error_quit(  msg:string );
begin
        write('Query Module: Error: ');
        writeln(msg);
        writeln('The Query Module is terminating your program.');
                halt(1);
end;

function orientation:char;

var
hr,hc: longint;
begin
hr := headingr;
hc := headingc;
orientation := '?';
        if((hr=-1) and (hc=0)) then orientation := 'N';
        if((hr=0) and  (hc=1)) then orientation := 'E';
        if((hr=1) and  (hc=0)) then orientation:= 'S';
        if((hr=0) and (hc=-1)) then orientation:= 'W';

end;


procedure Init;
var
infp:text;
i,j:longint;
c:char;
begin
{$I-}
    assign(infp,'xquery.in');
    reset(infp);
    if IOResult <> 0 then error_quit( 'Cannot open xquery.in' );

{$I-}
    assign(debugf,'query.log');
    rewrite(debugf);
    if IOResult <> 0 then error_quit( 'Cannot open query.log' );
writeln(debugf, 'Reading xquery.in...');
	flush(debugf);
    readln(infp,rows);
    if IOResult <> 0 then error_quit( 'Cannot read number of rows from file' );
    readln(infp,cols);
    if IOResult <> 0 then error_quit( 'Cannot read number of columns from file' );
    readln(infp,initialRow);
    if IOResult <> 0 then error_quit( 'Cannot read starting row from file' );
    readln(infp,initialCol);
    if IOResult <> 0 then error_quit( 'Cannot read starting column from file' );
    readln(infp,c);
    if IOResult <> 0 then error_quit( 'Cannot read starting orientation from file' );
    curRow := initialRow - 1;
    curCol := initialCol - 1;(* //change 1-based indexes to 0-based indexes*)
        headingr := 0;
        headingc := 0;
        if(upcase(c)='N') then headingr := -1;
        if(upcase(c)='S') then headingr := 1;
        if(upcase(c)='W') then headingc := -1;
        if(upcase(c)='E') then headingc := 1;
        if( (headingr=0) and (headingc=0) )then begin
                  writeln(debugf, 'Unrecognized orientation: ',c,'. (Use N,S,E or W)');
                  writeln(debugf,'Using N');
		  flush(debugf);
		  headingr:=-1; //use north
	end;

    for i:=0 to rows-1 do begin
        for j:=0 to cols-1 do begin
           read(infp,grid[i][j]);
           if IOResult<>0 then error_quit( 'Unable to read grid from file');
        end;
        readln(infp);
    end;
    if( grid[ curRow ][ curCol ] <> '.' ) then
       error_quit( 'Starting position must be empty');

    close( infp );
        writeln(debugf,'Game board:');
    for i:=0 to rows-1 do begin
        for j:=0 to cols-1 do begin
           write(debugf,grid[i][j]);

        end;
        writeln(debugf);
    end;

    writeln( debugf, 'Current position ',curRow+1, ',', curCol+1, ' facing ',orientation );
    flush( debugf );
    initialized := 1;
end;

function Forward:integer ;
(*//returns 1 if move is allowed, 0 if not allowed*)

var
 newRow,newCol:longint;
 begin
    if( initialized=0 ) then Init;
    writeln( debugf, 'Forward()' );
    flush( debugf );
    newRow := curRow+headingr;
    newCol := curCol+headingc;
    if( (newRow < 0) or (newCol < 0) or (newRow >= rows) or (newCol >= cols )) then begin
         writeln( debugf, 'returning 0 (',newRow+1,',',newCol+1,' is out of bounds)' );
         flush( debugf );
         Forward := 0;
    end
    else if( grid[ newRow ][ newCol ] <> '.' ) then begin
         writeln( debugf, 'returning 0 (',newRow+1,',',newCol+1,' has an obstacle)' );
         flush( debugf );
         Forward := 0;

    end
    else begin
    writeln( debugf, 'returning 1 (new position is ',newRow+1,',',newCol+1,' facing ', orientation,')' );
    flush( debugf );
    curRow := newRow;
    curCol := newCol;
    Forward:= 1;
    end;
end;

procedure Right;

var
  temp : longint;
begin
    if( initialized=0 ) then Init;
    temp := headingr;
    headingr := headingc;
    headingc := -temp;
    writeln( debugf, 'Right(): new orientation is ', orientation() );
    flush( debugf );
end;

procedure Left;

var
temp:longint;
begin
    if( initialized=0 ) then Init;
    temp := headingr;
    headingr := -headingc;
    headingc := temp;
    writeln( debugf, 'Left(): new orientation is ', orientation() );
    flush( debugf );

end;



begin
    initialized := 0;
end.
